@@ -25,7 +25,7 @@
  * $Id: nm-openvpn-service-openvpn-helper.c 4170 2008-10-11 14:44:45Z dcbw $
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -38,29 +38,56 @@
 #include <arpa/inet.h>
 #include <ctype.h>
 #include <netdb.h>
+#include <syslog.h>
 
-#include "nm-default.h"
-
-#include "nm-openvpn-service.h"
-#include "nm-utils.h"
+#include "nm-utils/nm-shared-utils.h"
+#include "nm-utils/nm-vpn-plugin-macros.h"
 
 extern char **environ;
 
-static gboolean helper_debug = FALSE;
+static struct {
+	int log_level;
+	const char *log_prefix_token;
+} gl;
+
+/*****************************************************************************/
+
+#define _NMLOG(level, ...) \
+	G_STMT_START { \
+		if (gl.log_level >= (level)) { \
+			g_print ("nm-openvpn[%s]: %-7s [helper-%ld] " _NM_UTILS_MACRO_FIRST (__VA_ARGS__) "\n", \
+			         gl.log_prefix_token ?: "???", \
+			         nm_utils_syslog_to_str (level), \
+			         (long) getpid () \
+			         _NM_UTILS_MACRO_REST (__VA_ARGS__)); \
+		} \
+	} G_STMT_END
+
+static inline gboolean
+_LOGD_enabled (void)
+{
+	return gl.log_level >= LOG_INFO;
+}
+
+#define _LOGD(...) _NMLOG(LOG_INFO,    __VA_ARGS__)
+#define _LOGI(...) _NMLOG(LOG_NOTICE,  __VA_ARGS__)
+#define _LOGW(...) _NMLOG(LOG_WARNING, __VA_ARGS__)
+
+/*****************************************************************************/
 
 static void
 helper_failed (GDBusProxy *proxy, const char *reason)
 {
 	GError *err = NULL;
 
-	g_warning ("nm-openvpn-service-openvpn-helper did not receive a valid %s from openvpn", reason);
+	_LOGW ("nm-openvpn-service-openvpn-helper did not receive a valid %s from openvpn", reason);
 
 	if (!g_dbus_proxy_call_sync (proxy, "SetFailure",
 	                             g_variant_new ("(s)", reason),
 	                             G_DBUS_CALL_FLAGS_NONE, -1,
 	                             NULL,
 	                             &err)) {
-		g_warning ("Could not send failure information: %s", err->message);
+		_LOGW ("Could not send failure information: %s", err->message);
 		g_error_free (err);
 	}
 
@@ -78,7 +105,7 @@ send_config (GDBusProxy *proxy, GVariant *config,
 	                             G_DBUS_CALL_FLAGS_NONE, -1,
 	                             NULL,
 	                             &err)) {
-		g_warning ("Could not send configuration information: %s", err->message);
+		_LOGW ("Could not send configuration information: %s", err->message);
 		g_error_free (err);
 		err = NULL;
 	}
@@ -89,7 +116,7 @@ send_config (GDBusProxy *proxy, GVariant *config,
 	                                     G_DBUS_CALL_FLAGS_NONE, -1,
 	                                     NULL,
 		                             &err)) {
-			g_warning ("Could not send IPv4 configuration information: %s", err->message);
+			_LOGW ("Could not send IPv4 configuration information: %s", err->message);
 			g_error_free (err);
 			err = NULL;
 		}
@@ -101,7 +128,7 @@ send_config (GDBusProxy *proxy, GVariant *config,
 	                                     G_DBUS_CALL_FLAGS_NONE, -1,
 	                                     NULL,
 		                             &err)) {
-			g_warning ("Could not send IPv6 configuration information: %s", err->message);
+			_LOGW ("Could not send IPv6 configuration information: %s", err->message);
 			g_error_free (err);
 			err = NULL;
 		}
@@ -142,28 +169,6 @@ addr4_to_gvariant (const char *str)
 	return g_variant_new_uint32 (temp_addr.s_addr);
 }
 
-static void
-parse_addr4_list (GPtrArray *array, const char *str)
-{
-	char **split;
-	int i;
-	struct in_addr temp_addr;
-
-	/* Empty */
-	if (!str || strlen (str) < 1)
-		return;
-
-	split = g_strsplit (str, " ", -1);
-	for (i = 0; split[i]; i++) {
-		if (inet_pton (AF_INET, split[i], &temp_addr) > 0)
-			g_ptr_array_add (array, g_variant_new_uint32 (temp_addr.s_addr));
-	}
-
-	g_strfreev (split);
-
-	return;
-}
-
 static GVariant *
 addr6_to_gvariant (const char *str)
 {
@@ -184,6 +189,30 @@ addr6_to_gvariant (const char *str)
 	return g_variant_builder_end (&builder);
 }
 
+static void
+parse_addr_list (GPtrArray *array4, GPtrArray *array6, const char *str)
+{
+	char **split;
+	int i;
+	GVariant *variant;
+
+	/* Empty */
+	if (!str || strlen (str) < 1)
+		return;
+
+	split = g_strsplit (str, " ", -1);
+	for (i = 0; split[i]; i++) {
+		if (array4 && (variant = addr4_to_gvariant (split[i])) != NULL)
+			g_ptr_array_add (array4, variant);
+		else if (array6 && (variant = addr6_to_gvariant (split[i])) != NULL)
+			g_ptr_array_add (array6, variant);
+	}
+
+	g_strfreev (split);
+
+	return;
+}
+
 static inline gboolean
 is_domain_valid (const char *str)
 {
@@ -216,14 +245,14 @@ get_ip4_routes (void)
 			break;
 
 		if (inet_pton (AF_INET, tmp, &network) <= 0) {
-			g_warning ("Ignoring invalid static route address '%s'", tmp ? tmp : "NULL");
+			_LOGW ("Ignoring invalid static route address '%s'", tmp ? tmp : "NULL");
 			continue;
 		}
 
 		snprintf (buf, BUFLEN, "route_netmask_%d", i);
 		tmp = getenv (buf);
 		if (!tmp || inet_pton (AF_INET, tmp, &netmask) <= 0) {
-			g_warning ("Ignoring invalid static route netmask '%s'", tmp ? tmp : "NULL");
+			_LOGW ("Ignoring invalid static route netmask '%s'", tmp ? tmp : "NULL");
 			continue;
 		}
 
@@ -231,7 +260,7 @@ get_ip4_routes (void)
 		tmp = getenv (buf);
 		/* gateway can be missing */
 		if (tmp && (inet_pton (AF_INET, tmp, &gateway) <= 0)) {
-			g_warning ("Ignoring invalid static route gateway '%s'", tmp ? tmp : "NULL");
+			_LOGW ("Ignoring invalid static route gateway '%s'", tmp ? tmp : "NULL");
 			continue;
 		}
 
@@ -244,7 +273,7 @@ get_ip4_routes (void)
 			errno = 0;
 			tmp_metric = strtol (tmp, NULL, 10);
 			if (errno || tmp_metric < 0 || tmp_metric > G_MAXUINT32) {
-				g_warning ("Ignoring invalid static route metric '%s'", tmp);
+				_LOGW ("Ignoring invalid static route metric '%s'", tmp);
 				continue;
 			}
 			metric = (guint32) tmp_metric;
@@ -300,13 +329,13 @@ get_ip6_routes (void)
 			errno = 0;
 			tmp_prefix = strtol (tmp, NULL, 10);
 			if (errno || tmp_prefix <= 0 || tmp_prefix > 128) {
-				g_warning ("Ignoring invalid static route prefix '%s'", tmp ? tmp : "NULL");
+				_LOGW ("Ignoring invalid static route prefix '%s'", tmp ? tmp : "NULL");
 				g_strfreev (dest_prefix);
 				continue;
 			}
 			prefix = (guint32) tmp_prefix;
 		} else {
-			g_warning ("Ignoring static route %d with no prefix length", i);
+			_LOGW ("Ignoring static route %d with no prefix length", i);
 			g_strfreev (dest_prefix);
 			continue;
 		}
@@ -317,7 +346,7 @@ get_ip6_routes (void)
 		route = nm_ip_route_new (AF_INET6, dest_prefix[0], prefix, tmp, -1, &error);
 		g_strfreev (dest_prefix);
 		if (!route) {
-			g_warning ("Ignoring a route: %s", error->message);
+			_LOGW ("Ignoring a route: %s", error->message);
 			g_error_free (error);
 			continue;
 		}
@@ -343,8 +372,8 @@ trusted_remote_to_gvariant (void)
 	if (tmp) {
 		val = addr6_to_gvariant (tmp);
 		if (val == NULL) {
-			g_warning ("%s: failed to convert VPN gateway address '%s' (%d)",
-			           __func__, tmp, errno);
+			_LOGW ("failed to convert VPN gateway address '%s' (%d)",
+			       tmp, errno);
 			return NULL;
 		}
 		return val;
@@ -354,7 +383,7 @@ trusted_remote_to_gvariant (void)
 	if (!tmp)
 		tmp = getenv ("remote_1");
 	if (!tmp) {
-		g_warning ("%s: did not receive remote gateway address", __func__);
+		_LOGW ("did not receive remote gateway address");
 		return NULL;
 	}
 
@@ -382,8 +411,8 @@ trusted_remote_to_gvariant (void)
 		hints.ai_flags = AI_ADDRCONFIG;
 		err = getaddrinfo (tmp, NULL, &hints, &result);
 		if (err != 0) {
-			g_warning ("%s: failed to look up VPN gateway address '%s' (%d)",
-			           __func__, tmp, err);
+			_LOGW ("failed to look up VPN gateway address '%s' (%d)",
+			       tmp, err);
 			return NULL;
 		}
 
@@ -405,15 +434,15 @@ trusted_remote_to_gvariant (void)
 		if (addr.s_addr != 0)
 			return g_variant_new_uint32 (addr.s_addr);
 		else {
-			g_warning ("%s: failed to convert or look up VPN gateway address '%s'",
-			           __func__, tmp);
+			_LOGW ("failed to convert or look up VPN gateway address '%s'",
+			       tmp);
 			return NULL;
 		}
 	} else {
 		val = addr4_to_gvariant (tmp);
 		if (val == NULL) {
-			g_warning ("%s: failed to convert VPN gateway address '%s' (%d)",
-			           __func__, tmp, errno);
+			_LOGW ("failed to convert VPN gateway address '%s' (%d)",
+			       tmp, errno);
 			return NULL;
 		}
 	}
@@ -431,7 +460,7 @@ main (int argc, char *argv[])
 	GVariant *val;
 	int i;
 	GError *err = NULL;
-	GPtrArray *dns_list;
+	GPtrArray *dns4_list, *dns6_list;
 	GPtrArray *nbns_list;
 	GPtrArray *dns_domains;
 	struct in_addr temp_addr;
@@ -441,6 +470,7 @@ main (int argc, char *argv[])
 	gboolean is_restart;
 	gboolean has_ip4_prefix = FALSE;
 	gboolean has_ip4_address = FALSE;
+	gboolean has_ip6_address = FALSE;
 	gchar *bus_name = NM_DBUS_SERVICE_OPENVPN;
 
 #if !GLIB_CHECK_VERSION (2, 35, 0)
@@ -452,19 +482,24 @@ main (int argc, char *argv[])
 			i++;
 			break;
 		}
-		if (!strcmp (argv[i], "--helper-debug"))
-			helper_debug = TRUE;
-		else if (!strcmp (argv[i], "--tun"))
+		if (nm_streq (argv[i], "--debug")) {
+			if (i + 2 >= argc) {
+				g_printerr ("Missing debug arguments (requires <LEVEL> <PREFIX_TOKEN>)\n");
+				exit (1);
+			}
+			gl.log_level = _nm_utils_ascii_str_to_int64 (argv[++i], 10, 0, LOG_DEBUG, 0);
+			gl.log_prefix_token = argv[++i];
+		} else if (!strcmp (argv[i], "--tun"))
 			tapdev = 0;
 		else if (!strcmp (argv[i], "--tap"))
 			tapdev = 1;
 		else if (!strcmp (argv[i], "--bus-name")) {
 			if (++i == argc) {
-				g_warning ("Missing bus name argument");
+				g_printerr ("Missing bus name argument\n");
 				exit (1);
 			}
 			if (!g_dbus_is_name (argv[i])) {
-				g_warning ("Invalid bus name");
+				g_printerr ("Invalid bus name\n");
 				exit (1);
 			}
 			bus_name = argv[i];
@@ -473,7 +508,7 @@ main (int argc, char *argv[])
 	}
 	shift = i - 1;
 
-	if (helper_debug) {
+	if (_LOGD_enabled ()) {
 		GString *args;
 
 		args = g_string_new (NULL);
@@ -487,13 +522,11 @@ main (int argc, char *argv[])
 			g_free (tmp);
 		}
 
-		g_message ("command line: %s", args->str);
+		_LOGD ("command line: %s", args->str);
 		g_string_free (args, TRUE);
-		g_message ("openvpn script environment ---------------------------");
-		iter = environ;
-		while (iter && *iter)
-			g_message ("%s", *iter++);
-		g_message ("------------------------------------------------------");
+
+		for (iter = environ; iter && *iter; iter++)
+			_LOGD ("environment: %s", *iter);
 	}
 
 	/* shift the arguments to the right leaving only those provided by openvpn */
@@ -511,7 +544,7 @@ main (int argc, char *argv[])
 	                                       NM_VPN_DBUS_PLUGIN_INTERFACE,
 	                                       NULL, &err);
 	if (!proxy) {
-		g_warning ("Could not create a D-Bus proxy: %s", err->message);
+		_LOGW ("Could not create a D-Bus proxy: %s", err->message);
 		g_error_free (err);
 		exit (1);
 	}
@@ -602,19 +635,25 @@ main (int argc, char *argv[])
 			g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_PREFIX, val);
 		}
 	} else
-		g_warning ("No IP4 netmask/prefix (missing or invalid 'ifconfig_netmask')");
+		_LOGW ("No IP4 netmask/prefix (missing or invalid 'ifconfig_netmask')");
 
 	val = get_ip4_routes ();
 	if (val)
 		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_ROUTES, val);
+	else if (is_restart) {
+		g_variant_builder_add (&ip4builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP4_CONFIG_PRESERVE_ROUTES,
+		                       g_variant_new_boolean (TRUE));
+	}
 
 	/* IPv6 address */
 	tmp = getenv ("ifconfig_ipv6_local");
 	if (tmp && strlen (tmp)) {
 		val = addr6_to_gvariant (tmp);
-		if (val)
+		if (val) {
 			g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_ADDRESS, val);
-		else
+			has_ip6_address = TRUE;
+		} else
 			helper_failed (proxy, "IP6 Address");
 	}
 
@@ -636,7 +675,7 @@ main (int argc, char *argv[])
 		errno = 0;
 		netbits = strtol (tmp, NULL, 10);
 		if (errno || netbits < 0 || netbits > 128) {
-			g_warning ("Ignoring invalid prefix '%s'", tmp);
+			_LOGW ("Ignoring invalid prefix '%s'", tmp);
 		} else {
 			val = g_variant_new_uint32 ((guint32) netbits);
 			g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_PREFIX, val);
@@ -646,10 +685,16 @@ main (int argc, char *argv[])
 	val = get_ip6_routes ();
 	if (val)
 		g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_ROUTES, val);
+	else if (is_restart) {
+		g_variant_builder_add (&ip6builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP6_CONFIG_PRESERVE_ROUTES,
+		                       g_variant_new_boolean (TRUE));
+	}
 
 	/* DNS and WINS servers */
 	dns_domains = g_ptr_array_sized_new (3);
-	dns_list = g_ptr_array_new ();
+	dns4_list = g_ptr_array_new ();
+	dns6_list = g_ptr_array_new ();
 	nbns_list = g_ptr_array_new ();
 
 	for (i = 1; i < 256; i++) {
@@ -668,18 +713,23 @@ main (int argc, char *argv[])
 		tmp += 12; /* strlen ("dhcp-option ") */
 
 		if (g_str_has_prefix (tmp, "DNS "))
-			parse_addr4_list (dns_list, tmp + 4);
+			parse_addr_list (dns4_list, dns6_list, tmp + 4);
 		else if (g_str_has_prefix (tmp, "WINS "))
-			parse_addr4_list (nbns_list, tmp + 5);
+			parse_addr_list (nbns_list, NULL, tmp + 5);
 		else if (g_str_has_prefix (tmp, "DOMAIN ") && is_domain_valid (tmp + 7))
 			g_ptr_array_add (dns_domains, tmp + 7);
 	}
 
-	if (dns_list->len) {
-		val = g_variant_new_array (G_VARIANT_TYPE_UINT32, (GVariant **) dns_list->pdata, dns_list->len);
+	if (dns4_list->len) {
+		val = g_variant_new_array (G_VARIANT_TYPE_UINT32, (GVariant **) dns4_list->pdata, dns4_list->len);
 		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_DNS, val);
 	}
 
+	if (has_ip6_address && dns6_list->len) {
+		val = g_variant_new_array (G_VARIANT_TYPE ("ay"), (GVariant **) dns6_list->pdata, dns6_list->len);
+		g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_DNS, val);
+	}
+
 	if (nbns_list->len) {
 		val = g_variant_new_array (G_VARIANT_TYPE_UINT32, (GVariant **) nbns_list->pdata, nbns_list->len);
 		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_NBNS, val);
@@ -688,9 +738,16 @@ main (int argc, char *argv[])
 	if (dns_domains->len) {
 		val = g_variant_new_strv ((const gchar **) dns_domains->pdata, dns_domains->len);
 		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_DOMAINS, val);
+
+		/* Domains apply to both IPv4 and IPv6 configurations */
+		if (has_ip6_address) {
+			val = g_variant_new_strv ((const gchar **) dns_domains->pdata, dns_domains->len);
+			g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_DOMAINS, val);
+		}
 	}
 
-	g_ptr_array_unref (dns_list);
+	g_ptr_array_unref (dns4_list);
+	g_ptr_array_unref (dns6_list);
 	g_ptr_array_unref (nbns_list);
 	g_ptr_array_unref (dns_domains);
 
@@ -702,7 +759,7 @@ main (int argc, char *argv[])
 		errno = 0;
 		mtu = strtol (tmp, NULL, 10);
 		if (errno || mtu < 0 || mtu > 20000) {
-			g_warning ("Ignoring invalid tunnel MTU '%s'", tmp);
+			_LOGW ("Ignoring invalid tunnel MTU '%s'", tmp);
 		} else {
 			val = g_variant_new_uint32 ((guint32) mtu);
 			g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_MTU, val);
